@@ -21,7 +21,7 @@
 /**
   TODO:
 
-   * maybe important, requires deeper code changes:
+   * maybe important, requires deeper changes to code or external interface:
    - download another file containing the DHCP/WLAN client counters
      - better get them merged into the same file
      - download/processing function evolved from synchronous
@@ -34,7 +34,7 @@
      - is g_unref() the right tool?
    - check return code from g_timeout_add_seconds/g_signal_connect
      - for now, we wouldn't know if start_timer()/install_handler() failed
-     - what to do in case of error?
+     - what to do in case of error? retry, abort, ignore?
    - does g_source_remove destroy the source or does it leak?
      - what data structures does a source use?
 
@@ -44,15 +44,13 @@
    - move constants from .h to configuration settings or autoconf options
 
    * style issues, don't affect features:
-   - use more HildonCaption in settings dialog?
-     - checkbox buttons look fine without Caption
-     - PickerButton could use a larger font
+   - PickerButton could use a larger font
 
    * nice to have:
    - draw nice icons
      - winni already did some work on new icons
    - support multiple instances
-     - to support future extensions
+     - for future extensions
      - not much use now
 
  **/
@@ -125,8 +123,12 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 {
 	g_assert(IS_CCCHH_DOORIS_DESKTOP_WIDGET(self));
 
-	if (!self->auto_update)
+	time_t timenow;
+
+	if (!self->auto_update) {
+		g_debug(WIDGET_NAME": start_update_timer(): auto_update disabled");
 		return FALSE;
+	}
 
 	if (!self->timeout_handler) {
 		g_debug(WIDGET_NAME": starting update timer");
@@ -137,7 +139,16 @@ start_update_timer(CCCHHDoorIsDesktopWidget *self)
 			g_warning(WIDGET_NAME": self->update_interval > UPDATE_INTERVAL_MAX, maybe a bug?");
 			self->update_interval = UPDATE_INTERVAL_MAX;
 		}
+
+		// force update if last poll is older than update interval
+		time (&timenow);
+		if (timenow != -1 && self->last_poll_ts && timenow - self->last_poll_ts > (time_t) self->update_interval*60) {
+			g_debug(WIDGET_NAME": start_update_timer(): last_poll expired, forcing update");
+			fetch_update_start(self);
+		}
+
 		self->timeout_handler = g_timeout_add_seconds(self->update_interval*60, ccchh_dooris_desktop_widget_on_timeout, self);
+
 		return TRUE;
 	}
 	return FALSE;
@@ -862,6 +873,7 @@ fetch_update_process (CCCHHDoorIsDesktopWidget *self, dooris_state_t new_state,
 		self->too_old = TRUE;
 		self->current_state = DOORIS_FNORD;
 	}
+	self->last_poll_ts = timenow;
 
 	if (self->current_state == self->last_state) {
 		g_debug(WIDGET_NAME": state unchanged");
@@ -950,7 +962,6 @@ ccchh_dooris_desktop_widget_on_timeout (gpointer data)
 	// connection event starts it again
 	if (!self->connected) {
 		g_debug(WIDGET_NAME": not connected, stopping update timer");
-		// TODO: do we need remove_source() here?
 		self->timeout_handler = 0;
 		force_redraw(self);
 
@@ -1321,7 +1332,7 @@ ccchh_dooris_desktop_widget_on_settings_dialog_response (GtkDialog *dialog,
 					force_redraw(self);
 				}
 
-				// start timer (if auto_update enabled, too)
+				// start timer (unless auto_update is disabled)
 				start_update_timer(self);
 			}
 
@@ -1664,13 +1675,6 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 	g_static_mutex_free(&(self->button_lock));
 	g_static_mutex_free(&(self->expose_lock));
 
-/*
-	if (self->gvfs_handle) {
-		g_object_unref(self->gvfs_handle);
-		self->gvfs_handle = NULL;
-	}
-*/
-
 	if (self->gvfs_input_buffer) {
 		g_free(self->gvfs_input_buffer);
 		self->gvfs_input_buffer = NULL;
@@ -1691,6 +1695,9 @@ ccchh_dooris_desktop_widget_finalize (GObject *object)
 		self->gconf_client = NULL;
 	}
 
+	// don't un-initialize GnomeVFS library here,
+	// it would render gvfs unusable for Hildon Deskop and other widgets
+
 	/* Call the base class's implementation: */
 	G_OBJECT_CLASS (ccchh_dooris_desktop_widget_parent_class)->finalize (object);
 }
@@ -1727,15 +1734,10 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 
 	g_debug(WIDGET_NAME": ccchh_dooris_desktop_widget_init()");
 
-/*
-	// initialize GnomeVFS library
-	if (!gnome_vfs_init()) {
-		// FIXME: crashes the whole desktop process
-		g_error(WIDGET_NAME": could not initialize GnomeVFS library");
-		abort();
-	}
-*/
- 
+	// don't initialize GnomeVFS library here, Hildon did it already
+    // TODO: - can we rely on that?
+	//       - would it hurt to call gnome_vfs_init() anyway?
+
 	// default settings
 	self->notify_enabled = TRUE;
 	self->update_interval = 23;
@@ -1757,6 +1759,7 @@ ccchh_dooris_desktop_widget_init (CCCHHDoorIsDesktopWidget *self)
 
 	self->last_change_ts = 0L;
 	self->last_report_ts = 0L;
+	self->last_poll_ts = (time_t) 0L;
 	self->too_old = FALSE;
 
 	g_static_mutex_init(&(self->fetch_lock));
